function out = get(a,property)
% function Out = get(A,Prop)
%
% DESCRIPTION 
%   Get polydecvar properties.
%   
% INPUTS 
%   A: polydecvar 
%   Property: polydecvar property
%
% OUTPUTS  
%   Out: Value of the property 
%  
% SYNTAX 
%   Out = get(A,Property);

% 6/7/2002: Written by Packard
% 10/30/2002 PJS  Make PVget act on pmat
  
GSp = PVget(a,'PropNames');
if nargin==2
    % Use CaseInsensitivePartialMatch to find the correct property name
    [idx,fstr] = PVcipmatch(GSp.GPropNames,property);
    if isempty(idx)
        % Try to get from polynomial object
        ap = a.polynomial;
        try
            out = get(ap,property);
        catch
            error(['Property ' property ' is not gettable']);
        end
    else
        % Extract value with PVGET
        out = PVget(a,fstr);
    end
else
    % Create an empty STRUCT, with all the entries from GProp as
    % fieldsnames
    nprop = length(GSp.GPropNames);
    %plist = [GSp.GPropNames cell(nprop,1)]';
    %tout = struct(plist{:});

    % Get polynomial properties
    ap = a.polynomial;
    tout = get(ap);
    
    % Fill each field with the appropriate value
    for i=1:nprop
        tout = setfield(tout,GSp.GPropNames{i}, ...
        PVget(a,GSp.GPropNames{i}));
    end
    
    
    if nargout==1
        out = tout;
    else
        % If there are no output arguments, simply display the result 
        disp(tout);
    end
end


