% SOSOPTDEMO2 --- Lyapunov Function Search 
% Section 3.2 of SOSTOOLS User's Manual
% 


% =============================================
% Define polynomial variables
pvar x1 x2 x3;       
x = [x1;x2;x3];

% =============================================
% Constructing the vector field 
%     dx/dt = f/(x3^2+1)
f = [(-x1^3-x1*x3^2)*(x3^2+1);
    (-x2-x1^2*x2)*(x3^2+1);
    (-x3+3*x1^2*x3)*(x3^2+1)-3*x3];


% =============================================
% The Lyapunov function V(x): 
zV = [x1^2; x2^2; x3^2];
[V,cV] = polydecvar('c',zV,'vec');

% =============================================
% Next, define SOSP constraints
sosconstr = cell(2,1);

% Constraint 1 : V(x) - (x1^2 + x2^2 + x3^2) >= 0
L1 = x'*x;
sosconstr{1} = V-L1;

% Constraint 2: -dV/dx*(x3^2+1)*f >= 0
sosconstr{2} = -jacobian(V,x)*f;

% =============================================
% Call sosopt to find a Lyapunov function
% Either kernel or image form can be used.
opts = [];
opts.form = 'image';
%opts.form = 'kernel';

[info,dopt,sossol] = sosopt(sosconstr,x,opts);

% =============================================
% Finally, get solution
fprintf('\n----------------Results');
if info.feas
    fprintf('\nSOS Optimization is feasible.\n');
    Vs = subs(V,dopt)
else
    fprintf('\nSOS Optimization is not feasible. \n');    
    return
end

% =============================================
% Verify the Gram Marix Decompositions in sossol
w1 = sossol{1,2};
Q1 = sossol{1,3};
e1 = (Vs-L1) - (w1'*Q1*w1);
fprintf('\nMax magnitude coefficient of s(1)-w1''*Q1*w1 is:')
disp(full(max(abs(e1.coefficient))))
fprintf('Minimum eigenvalue of Q1 is:')
disp(min(eig(Q1)));

w2 = sossol{2,2};
Q2 = sossol{2,3};
e2 = (-jacobian(Vs,x)*f) - (w2'*Q2*w2);
fprintf('\nMax magnitude coefficient of s(2)-w2''*Q2*w2 is:')
disp(full(max(abs(e2.coefficient))))
fprintf('Minimum eigenvalue of Q2 is:')
disp(min(eig(Q2)));


