% SOSOPTDEMO3 --- Bound on Global Extremum
% Section 3.3 of SOSTOOLS User's Manual
% 

% =============================================
% Define polynomial variables
pvar x1 x2 gam;
x = [x1;x2];

% =============================================
% Next, define SOS constraint

% Constraint : r(x)*(f(x) - gam) >= 0
% f(x) is the Goldstein-Price function
f1 = x1+x2+1;
f2 = 19-14*x1+3*x1^2-14*x2+6*x1*x2+3*x2^2;
f3 = 2*x1-3*x2;
f4 = 18-32*x1+12*x1^2+48*x2-36*x1*x2+27*x2^2;

f = (1+f1^2*f2)*(30+f3^2*f4);

sosconstr = f-gam;


% =============================================
% Set objective : maximize gam
obj = -gam;

% =============================================
% And call solver
% Either kernel or image form can be used.
opts = [];
opts.form = 'image';
%opts.form = 'kernel';
    
[info,dopt,sossol] = sosopt(sosconstr,x,obj,opts);

% =============================================
% Finally, get solution
fprintf('\n----------------Results');
if info.feas
    fprintf('\nSOS Optimization is feasible:\n');
    SOLgamma = subs(gam,dopt)
else
    fprintf('\nSOS Optimization is not feasible. \n');    
    return
end

% =============================================
% Verify the Gram Marix Decomposition in sossol
w1 = sossol{2};
Q1 = sossol{3};
e1 = (f-SOLgamma) - (w1'*Q1*w1);
fprintf('\nMax magnitude coefficient of s(1)-w1''*Q1*w1 is:')
disp(full(max(abs(e1.coefficient))))
fprintf('Minimum eigenvalue of Q1 is:')
disp(min(eig(Q1)));




