% SOSOPT LP Demo
% Create a problem with both SOS and LP constraints on the decision vars

% Create polynomial variables, x, and decision variables, d
pvar d1 d2 x1 x2;
x = [x1;x2];
d = [d1;d2];

% Create an SOS constraint on the coefficients of s
s = d1*x1^2+4*x1*x2+d2*x2^2;

% Create an LP constraint on the coefficients
lpconstr.A = [-1 0; 1 0; 0 -1; 0 1];
lpconstr.b = [-1; 4; -1; 4];
lpconstr.d = d;

% Create objective function
th = pi/4;
c = [cos(th); sin(th)];
obj = c'*d;

% Solve the problem with sosopt
% Either kernel or image form can be used.
opts = [];
opts.form = 'image';
%opts.form = 'kernel';

[info,dopt,sossol] = sosopt(s,x,obj,lpconstr,opts);

% Get solution
fprintf('\n----------------Results');
if info.feas
    fprintf('\nSOS/LP Optimization is feasible:\n');
    d1sol = double(subs(d1,dopt))
    d2sol = double(subs(d2,dopt))
else
    fprintf('\nSOS/LP Optimization is not feasible. \n');    
    return
end

% Plot constraints, objective function, and optimal solution
figure(1);
plot([1 4 4 1 1],[1 1 4 4 1],'b'); hold on;
x1dat = linspace(0.5,5);
x2dat = 4./x1dat;
plot(x1dat,x2dat,'r');
quiver(d1sol,d2sol,c(1),c(2),'k');
plot(d1sol,d2sol,'ko'); hold off;
legend('LP Constraints','SOS Constraint','Objective Dir.',....
    'Optimal Pt', 'Location','SouthWest')
xlabel('d1');
ylabel('d2');
axis([0.5 4.5 0.5 4.5])
